// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "codecompletiondebug.h"


Q_LOGGING_CATEGORY(KDEV_PYTHON_CODECOMPLETION, "kdevelop.plugins.python.codecompletion", QtInfoMsg)

