/********************************************************************************
** Form generated from reading UI file 'pep8.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PEP8_H
#define UI_PEP8_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_pep8
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *enableChecking;
    QFormLayout *formLayout;
    QLineEdit *disableErrors;
    QLabel *label_2;
    QLineEdit *enableErrors;
    QLabel *label_3;
    QSpinBox *maxLineLength;
    QLabel *label;
    QLabel *label_4;

    void setupUi(QWidget *pep8)
    {
        if (pep8->objectName().isEmpty())
            pep8->setObjectName("pep8");
        pep8->resize(717, 298);
        verticalLayout = new QVBoxLayout(pep8);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        enableChecking = new QGroupBox(pep8);
        enableChecking->setObjectName("enableChecking");
        enableChecking->setCheckable(true);
        formLayout = new QFormLayout(enableChecking);
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        disableErrors = new QLineEdit(enableChecking);
        disableErrors->setObjectName("disableErrors");
        disableErrors->setReadOnly(false);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, disableErrors);

        label_2 = new QLabel(enableChecking);
        label_2->setObjectName("label_2");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, label_2);

        enableErrors = new QLineEdit(enableChecking);
        enableErrors->setObjectName("enableErrors");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, enableErrors);

        label_3 = new QLabel(enableChecking);
        label_3->setObjectName("label_3");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, label_3);

        maxLineLength = new QSpinBox(enableChecking);
        maxLineLength->setObjectName("maxLineLength");
        maxLineLength->setMinimum(40);
        maxLineLength->setMaximum(1000);
        maxLineLength->setSingleStep(1);
        maxLineLength->setValue(80);

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, maxLineLength);

        label = new QLabel(enableChecking);
        label->setObjectName("label");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label);

        label_4 = new QLabel(enableChecking);
        label_4->setObjectName("label_4");
        QFont font;
        font.setItalic(true);
        label_4->setFont(font);

        formLayout->setWidget(0, QFormLayout::ItemRole::SpanningRole, label_4);


        verticalLayout->addWidget(enableChecking);


        retranslateUi(pep8);

        QMetaObject::connectSlotsByName(pep8);
    } // setupUi

    void retranslateUi(QWidget *pep8)
    {
        enableChecking->setTitle(tr2i18n("Enable Style Checking", nullptr));
        disableErrors->setPlaceholderText(tr2i18n("Comma-separated list of error codes", nullptr));
        label_2->setText(tr2i18n("Enable these errors and warnings:", nullptr));
        enableErrors->setPlaceholderText(tr2i18n("Comma-separated list of error codes", nullptr));
        label_3->setText(tr2i18n("Maximum line length:", nullptr));
        label->setText(tr2i18n("Disable these errors and warnings:", nullptr));
        label_4->setText(tr2i18n("The `pycodestyle` Python module must be installed in order to use style checking.", nullptr));
        (void)pep8;
    } // retranslateUi

};

namespace Ui {
    class pep8: public Ui_pep8 {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PEP8_H

