/********************************************************************************
** Form generated from reading UI file 'projectconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROJECTCONFIG_H
#define UI_PROJECTCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_ProjectConfig
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *label;
    QLineEdit *pythonInterpreter;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *ProjectConfig)
    {
        if (ProjectConfig->objectName().isEmpty())
            ProjectConfig->setObjectName("ProjectConfig");
        ProjectConfig->resize(740, 479);
        verticalLayout = new QVBoxLayout(ProjectConfig);
        verticalLayout->setObjectName("verticalLayout");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        label = new QLabel(ProjectConfig);
        label->setObjectName("label");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label);

        pythonInterpreter = new QLineEdit(ProjectConfig);
        pythonInterpreter->setObjectName("pythonInterpreter");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, pythonInterpreter);


        verticalLayout->addLayout(formLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        retranslateUi(ProjectConfig);

        QMetaObject::connectSlotsByName(ProjectConfig);
    } // setupUi

    void retranslateUi(QWidget *ProjectConfig)
    {
        label->setText(tr2i18n("Python interpreter:", nullptr));
        pythonInterpreter->setPlaceholderText(tr2i18n("Full path to python interpreter, leave empty for default", nullptr));
        (void)ProjectConfig;
    } // retranslateUi

};

namespace Ui {
    class ProjectConfig: public Ui_ProjectConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PROJECTCONFIG_H

